package com.abmyers.utils;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.abmyers.spinfw.action.ActionContext;

public class BeanUtils {

	public final static Logger logger = Logger.getLogger(BeanUtils.class
			.getName());

	/**
	 * Populate an objects properties from matching request parameters
	 *
	 * @param bean
	 * @param request
	 */
	public static void populateBean(Object bean, ActionContext ac) {

		Method[] methods = bean.getClass().getMethods();

		for (Method m : methods) {
			String methodNameLower = m.getName().toLowerCase();
			if (m.getName().toLowerCase().startsWith("set")) {
				String assumedFieldName = methodNameLower.substring(3);
				if (assumedFieldName.length() > 0 && ac.containsParameterNoCase(assumedFieldName)) {
					// call the setter, and pass in the value
					try {
						// TODO: Make this work for arrays
						// need to convert to the correct type
						Object paramConverted = BeanUtils.coerceParameter(ac.getParameterNoCase(assumedFieldName), m.getParameterTypes()[0]);
						m.invoke(bean, paramConverted);
					} catch (IllegalArgumentException e) {
						logger.log(Level.INFO, e.getMessage(), e);
					} catch (IllegalAccessException e) {
						logger.log(Level.INFO, e.getMessage(), e);
					} catch (InvocationTargetException e) {
						logger.log(Level.INFO, e.getMessage(), e);
					}
				}
			}
		}

	}

	private static Object coerceParameter(String parameterNoCase, Class<?> class1) {
		if (class1.getName().equals("java.lang.String")) {
			return parameterNoCase;
		} else if (class1.getName().equals("java.lang.Long")) {
			try {
				return Long.parseLong(parameterNoCase);
			} catch (NumberFormatException nfe) {
				return null;
			}
		} else if (class1.getName().equals("java.lang.Integer")) {
			try {
				return Integer.parseInt(parameterNoCase);
			} catch (NumberFormatException nfe) {
				return null;
			}
		} else {
			return null;
		}
	}

}
