package com.abmyers.validation;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Validator {

	private List<ValidationContext> presenceList;
	private List<ValidationContext> formatList;
	private static final String PRESENCE_MESSAGE = "must be provided";
	private static final String NUMERICALITY_MESSAGE = "is not numeric";	
	private static final String CURRENCY_MESSAGE = "is not a valid amount";	

	public final static Logger logger = Logger.getLogger(Validator.class
			.getName());

	public Validator() {
		presenceList = new ArrayList<ValidationContext>();
		formatList = new ArrayList<ValidationContext>();
	}

	public void validatesPresenceOf(String fieldName) {
		presenceList.add(new ValidationContext(fieldName, PRESENCE_MESSAGE));
	}

	public void validatesPresenceOf(String fieldName, String message) {
		presenceList.add(new ValidationContext(fieldName, message));
	}

	public void validatesNumericalityOf(String fieldName, int decimalPlaces) {
		this.validatesNumericalityOf(fieldName, NUMERICALITY_MESSAGE, decimalPlaces);
	}

	public void validatesNumericalityOf(String fieldName, String message, int decimalPlaces) {
		
		String pattern = "[0-9]+";
		if (decimalPlaces > 0) {
			pattern = pattern + "\\.[0-9]+{" + decimalPlaces + "}";
		}		
		formatList.add(new ValidationContext(fieldName, message, pattern));		
	}

	public void validatesCurrencyFormat(String fieldName) {
		this.validatesCurrencyFormat(fieldName, CURRENCY_MESSAGE);
	}
	
	public void validatesCurrencyFormat(String fieldName, String message) {
		validatesNumericalityOf(fieldName, message, 2);
	}
	
	public void validatesFormatOf(String fieldName, String message, String regexp) {
		formatList.add(new ValidationContext(fieldName, message, regexp));
	}
	
	public Map<String, String> validate(final Object o) {
		Map<String, String> errors = new HashMap<String, String>();
		for (ValidationContext v : presenceList) {
			doValidatePresenceOf(o, v.getFieldName(), v.getErrorMsg(), errors);
		}
		
		for (ValidationContext v : formatList) {
			doValidateFormatOf(o, v.getFieldName(), v.getErrorMsg(), errors, v.getRegexp());
		}
		
		return errors;
	}

	private void doValidatePresenceOf(final Object o, final String fieldName,
			final String message, Map<String, String> errors) {
		Object result = callGetter(o, fieldName);
		String resultStr = null;
		if (result != null) {
			resultStr = result.toString();
		}
		if (resultStr == null || resultStr.length() == 0) {
			errors.put("fieldName", message);
		}
	}
	
	private void doValidateFormatOf(final Object o, final String fieldName, 
			final String message, Map<String, String> errors, String withPattern) {
				
		Object result = callGetter(o, fieldName);
		String resultStr = null;
		if (result != null) {
			resultStr = result.toString();
		}
		if (resultStr == null || resultStr.length() == 0) {
			errors.put(fieldName, message);
		} else {
			Pattern p = Pattern.compile(withPattern);
			Matcher m = p.matcher(resultStr);
			if (!m.matches()) {
				errors.put(fieldName, message);
			}
		}
	}
		
	/*************************************************************************
	 * PRIVATE METHODS
	 *************************************************************************/
	private Object callGetter(Object o, String fieldName) {
		Object result = null;

		Class<?> c = o.getClass();

		String methodName = "get" + fieldName.substring(0, 1).toUpperCase()
				+ fieldName.substring(1);
		try {
			Method method = c.getMethod(methodName);
			result = method.invoke(o);
		} catch (Exception e) {
			logger.log(Level.INFO, "validateIsInteger failed with error {1}",
					e.getMessage());
		}

		return result;
	}
}
